<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns="http://www.w3.org/1999/xhtml">

<!--
 
 Service Report (HTML)
 Copyright (c) 2003 Star Chamber Inc.
 
-->

<xsl:output
	method="html" indent="yes" encoding="iso-8859-1"
	doctype-public="-//W3C//DTD XHTML 1.0 Strict//EN"
	doctype-system="http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd"/>
<xsl:strip-space elements="*"/>

<!-- ============================ Report Style Sheet =============================== -->

<xsl:template name="sussen-report-style">
	<style type="text/css">
		<xsl:comment>

		body { 
			background-color: #ffffff; 
   			font: 10pt Verdana, Geneva, Arial, Helvetica, sans-serif;
			margin:10px 10px 0px 10px;
			padding:0px;
		}

		p {
			font: 10pt Verdana, Geneva, Arial, Helvetica, sans-serif;
			margin-top: 0px;
			text-align: justify;
		}

		h2 { 
			background: #99cccc;
			color: Black;
		}

		#container {
			margin: 0px;
			padding: 0px 175px 0px 110px
		}

		#intro {
			min-width: 470px;
		}
		
		#pageHeader {
			margin-bottom: 20px; 
		}
	
		#summary {
			margin: 20px 20px 20px 10px; 
		}

		#leftcontent {
			margin-right:15px;
			padding: 0px 10px 0px 10px;
		}

		#rightcontent {
			float: right;
			width: 50%;
			padding: 0px 10px 0px 10px;
		}

		#normal {
			width: 100%;
			float: left;
		}

		#footer {
			text-align: center;
			padding-top: 20px;
		}

	</xsl:comment>
	</style>
</xsl:template>

<xsl:template match="/">
<html>
	<head>
		<title>Sussen Service Report</title>
		<xsl:call-template name="sussen-report-style"/>
	</head>
	<body>
	<div id="container">
		<xsl:apply-templates select="report"/>
	</div>
	</body>
</html>
</xsl:template>

<!-- ============================ Report Header =============================== -->

<xsl:template name="sussen-report-header">

	<div id="intro">	
		<div id="pageHeader">
			<h2><span>Service Report</span></h2>
		</div>
		
		<div id="summary">
			<p class="bodycopy">This report gives details on the network services
			that were found running during the scan.</p>
		</div>	
	</div>

</xsl:template>

<!-- ============================ Service Details =============================== -->


<xsl:template match="result" mode="toc">

	<h2><span>Service Details</span></h2>
		
	<xsl:variable name="hostname">
		<xsl:value-of select="host/@name"/>
	</xsl:variable>

	<div id="leftcontent">
		host: <xsl:value-of select="$hostname"/>
	</div>

	<xsl:for-each select="ports/port">
					
		<div id="rightcontent">
		<xsl:value-of select="./service/@name"/>			
		<xsl:if test="./@portid">				
				<xsl:text>(</xsl:text>
				<xsl:value-of select="./@portid"/><xsl:text>/</xsl:text>
				<xsl:value-of select="./@protocol"/><xsl:text>)</xsl:text>
		</xsl:if>
        </div>
	</xsl:for-each>
</xsl:template>

<xsl:template match="results">
 <xsl:apply-templates select="." mode="analysis"/>
</xsl:template>

<xsl:template match="results" mode="analysis">
	<xsl:for-each select="result">
		<xsl:apply-templates select="." mode="toc"/>
	</xsl:for-each>
</xsl:template>

<xsl:template match="report">
   <xsl:call-template name="sussen-report-header"/>
   <xsl:apply-templates select="results"/>
   
   <div id="normal">
   <div id="footer">
   Report generated by <a href="http://sussen.sourceforge.net/">Sussen</a>
   </div>
   </div>
   
</xsl:template>


</xsl:stylesheet>
