/* 
 * Copyright (C) 2003 Star Chamber Inc.
 *
 * Sussen - sussen-fileio.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Author: Loren Bandiera <lorenb@starchamber.ca>
 */
 
#include "sussen.h"
#include "sussen-fileio.h"
#include "sussen-preferences.h"
#include "sussen-util.h"
#include <glib.h>

void create_dot_sussen (void) {	
	
	gchar *path = get_dot_sussen_path();
		
	if (mkdir (g_strdup(path),S_IRUSR|S_IWUSR|S_IXUSR) != 0) {		
		g_print ("Unable to create: [%s]\n", path);
		exit (-1);
	}

	g_free (path);
}

gchar* get_backend_xml_filename (void) {

	gchar *dot_sussen;
	gchar *ret;

	dot_sussen = get_dot_sussen_path ();
	if (dot_sussen) {
		ret = g_strdup_printf ("%s/backend.xml", dot_sussen);
		g_free (dot_sussen);
		return ret;	
	}
	
	return NULL;
}


gchar* get_server_xml_filename (void) {

	gchar *dot_sussen;
	gchar *ret;

	dot_sussen = get_dot_sussen_path ();
	if (dot_sussen) {
		ret = g_strdup_printf ("%s/server.xml", dot_sussen);
		g_free (dot_sussen);
		return ret;	
	}
	
	return NULL;
}

gchar *get_dot_sussen_path (void) {
	
	gchar *ret;
  
	ret = g_strdup_printf ("%s/.sussen", g_get_home_dir ());
	if (ret) {
		return(ret);
	} 

	return(NULL);	
}

gboolean dot_sussen_there (void) {

	GDir *dot_sussen = NULL;
	GError *err = NULL;
	gchar *path_to_dot_sussen = get_dot_sussen_path();
	
	dot_sussen = g_dir_open (path_to_dot_sussen, 0, &err);
	if (err) {
		g_free (path_to_dot_sussen);
		g_free (err);
		return FALSE;
	}
	
	g_free (path_to_dot_sussen);
	return TRUE;
}

void list_callback (gpointer data, gpointer user_data) {
	
	gchar *tmp;
	gchar *host;
	gchar *port;
	gchar *user;
	gchar *enabled;
	xmlDocPtr doc;
	xmlNodePtr child;
	
	doc = (xmlDocPtr) user_data;
	
	tmp = g_strdup ((gchar*)data);
	host = strtok (tmp, ":");  
	port = strtok (NULL, ":");
	user = strtok (NULL, ":");
	enabled = strtok (NULL, ":");

	child = xmlNewChild(doc->children, NULL, "server", NULL); 
	xmlSetProp (child, "name", host);
	xmlSetProp (child, "port", port);
	xmlSetProp (child, "user", user);
	xmlSetProp (child, "enabled", enabled);
}

gchar* get_backend_xml (void) {
	
	xmlDocPtr doc;
	xmlNodePtr cur;
	gchar *dsn = NULL;
	gchar *user = NULL;
	gchar *pass = NULL;
	gchar *ret = NULL;
	
	gchar *filename = get_backend_xml_filename ();
	doc = xmlParseFile(filename);
	if (doc == NULL) {
		return NULL;
	}
	
    cur = xmlDocGetRootElement(doc);
    if (cur == NULL) {
		return NULL;		
    }	
		
	cur = cur->xmlChildrenNode;
	while (cur != NULL) {
		
		dsn = xmlGetProp(cur, (const xmlChar *) "dsn");
		user = xmlGetProp(cur, (const xmlChar *) "user");
		pass = xmlGetProp(cur, (const xmlChar *) "pass");
		cur = cur->next;
	}

	ret = g_strdup_printf ("%s:%s:%s", dsn, user, pass);
	return ret;
}

void set_backend_xml(const gchar *dsn, const gchar *user, const gchar *pass) {

	xmlDocPtr doc;
	xmlNodePtr child;
	
	gchar *filename = get_backend_xml_filename ();

	doc = xmlNewDoc("1.0");
	doc->children = xmlNewDocNode (doc, NULL, "backends", NULL);
	child = xmlNewChild(doc->children, NULL, "backend", NULL); 
	xmlSetProp (child, "dsn", dsn);
	xmlSetProp (child, "user", user);
	xmlSetProp (child, "pass", pass);
	
	xmlSaveFile (filename,doc);	
	
	xmlFreeDoc(doc);
	g_free (filename);
}

void set_server_xml (GSList *list) {

	xmlDocPtr doc;

	gchar *filename = get_server_xml_filename ();

	doc = xmlNewDoc("1.0");
	doc->children = xmlNewDocNode (doc, NULL, "servers", NULL);
	g_slist_foreach  (list, list_callback, (gpointer)doc);
	
	xmlSaveFile (filename,doc);	
	
	xmlFreeDoc(doc);
	g_free (filename);
}


GSList* get_server_xml (void) {
	
	GSList *server_list = NULL;
	xmlDocPtr doc;
	xmlNodePtr cur;
	const gchar *filename;
	
	filename = get_server_xml_filename ();
	doc = xmlParseFile(filename);
	if (doc == NULL) {
		sussen_error_dialog (_("could not find server.xml"));
		return NULL;
	}
	
    cur = xmlDocGetRootElement(doc);
    if (cur == NULL) {
		sussen_error_dialog (_("corrupt or invalid server.xml"));
		xmlFreeDoc(doc);
		return NULL;		
    }	
		
	cur = cur->xmlChildrenNode;
	
	while (cur != NULL) {

		gchar *name;
		gchar *port;
		gchar *user;
		gchar *enabled;
		gchar *list_str;
		
		list_str = g_malloc (255);
		
		name = xmlGetProp(cur, (const xmlChar *) "name");
		port = xmlGetProp(cur, (const xmlChar *) "port");
		user = xmlGetProp(cur, (const xmlChar *) "user");
		enabled = xmlGetProp(cur, (const xmlChar *) "enabled");
		
		g_snprintf (list_str, 255, "%s:%s:%s:%s", name, port, user, enabled);
		server_list = g_slist_append (server_list, g_strdup(list_str));
		
		g_free (list_str);	
		cur = cur->next;
	}
	
	return server_list;	
}

gchar* get_reports_xml_filename (void) {

	gchar *dot_sussen;
	gchar *ret;

	dot_sussen = get_dot_sussen_path ();
	if (dot_sussen) {
		ret = g_strdup_printf ("%s/reports.xml", dot_sussen);
		g_free (dot_sussen);
		return ret;	
	}
	
	return NULL;	
}

GSList* get_reports_xml (void) {

	GSList *reports_list = NULL;
	xmlDocPtr doc;
	xmlNodePtr cur;
	const gchar *filename;
	
	filename = get_reports_xml_filename ();
	doc = xmlParseFile(filename);
	if (doc == NULL) {
		sussen_error_dialog (_("could not find reports.xml"));
		return NULL;
	}
	
    cur = xmlDocGetRootElement(doc);
    if (cur == NULL) {
		sussen_error_dialog (_("corrupt or invalid reports.xml"));

		xmlFreeDoc(doc);
		return NULL;		
    }	
		
	cur = cur->xmlChildrenNode;
	
	while (cur != NULL) {

		gchar *name;
		gchar *file;
		gchar *format;
		
		
		name = xmlGetProp(cur, (const xmlChar *) "name");
		file = xmlGetProp(cur, (const xmlChar *) "file");
		format = xmlGetProp(cur, (const xmlChar *) "format");
		
		gchar *list_str = g_strdup_printf ("%s:%s:%s", name, file, format);
		reports_list = g_slist_append (reports_list, g_strdup(list_str));
		
		g_free (list_str);	
		cur = cur->next;
	}
	
	return reports_list;	
}

gint get_server_cert_hash (const gchar *sname, guchar *hash) {
/*
	xmlDocPtr doc;
	xmlNodePtr cur;
	gchar *filename;
	gint i,x;

	gchar *tmp = get_dot_sussen_path ();
	filename = g_strdup_printf ("%s/server.cert.xml", tmp);
	doc = xmlParseFile(filename);
	if (doc == NULL) {
		return -1;
	}
	
    cur = xmlDocGetRootElement(doc);
    if (cur == NULL) {
		return -1;		
    }	
		
	cur = cur->xmlChildrenNode;
	while (cur != NULL) {

		gchar *name;
		gchar *stored_hash;
		
		name = xmlGetProp(cur, (const xmlChar *) "name");
		stored_hash = xmlGetProp(cur, (const xmlChar *) "hash");
		
		if (name && stored_hash) {
			if (g_ascii_strncasecmp(name,sname,strlen(sname)) == 0) {
				for (i = 0; i < SHA_DIGEST_LENGTH; i ++) {
					(void) sscanf(stored_hash + 2 * i, "%2x", &x);
					hash[i] = x;
				}
				return 1;
			}			
		}

		cur = cur->next;
	}

	return 0;	
*/
	return 0;
}


gint set_server_cert_hash (const gchar *sname, guchar *hash) {
/*
	xmlDocPtr doc;
	xmlNodePtr cur;
	xmlNodePtr new_child;
	gchar *filename;
	gchar str_hash[1024];
	gint i;

	gchar *tmp = get_dot_sussen_path ();
	filename = g_strdup_printf ("%s/server.cert.xml", tmp);
	doc = xmlParseFile(filename);
	if (doc == NULL) {
		return -1;
	}
	
    cur = xmlDocGetRootElement(doc);
    if (cur == NULL) {
		return -1;		
    }	
	
	new_child = xmlNewTextChild (cur, NULL, "cert", NULL);
	xmlNewProp (new_child, "name", sname);

	for (i = 0; i < SHA_DIGEST_LENGTH; i ++)
		sprintf(str_hash + 2 * i, "%02x", hash[i]);

	xmlNewProp (new_child, "hash", str_hash);
	xmlSaveFormatFile (filename, doc, 1);	
*/
	return 0;
}
