/* 
 * Copyright (C) 2003 Star Chamber Inc.
 *
 * Sussen - sussen-main.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Author: Loren Bandiera <lorenb@starchamber.ca>
 */

#include "sussen.h"
#include "sussen-backend.h"
#include "sussen-fileio.h"
#include "sussen-getting-started-druid.h"
#include "sussen-main-callback.h"
#include "sussen-network.h"
#include "sussen-util.h"
#include "sussen-xml.h"
#include <libxslt/xslt.h>
#include <libgnomedb/libgnomedb.h>
#include <libgnomedb/gnome-db-dsn-config-druid.h>

/* Nessus global variables */

gint F_nessusd_running;
gint First_time = 0;

/* 
 * Sussen globals 
 */

GtkWidget *main_window;
GladeXML *mainwindow_xml;

GtkWidget *sensor_tree;
GtkListStore *sensor_store;

GtkWidget *targets_tree;
GtkListStore *targets_store;

GtkWidget *scan_status_tree;
GtkListStore *scan_status_store;

GtkTreeModel *global_model;
GtkTreeIter *global_iter;

gint use_policy_file = 0;
GHashTable *plugin_hash;

GTcpSocket *global_gtcpsocket = NULL;
gboolean sussen_detached_mode;
gboolean sussen_save_sessions;
gboolean sussen_save_detached_sessions;

GData *server_preferences;
GData *server_information;
GSList *plugin_list = NULL;
GSList *server_rules = NULL;
GData *plugin_depends;
GHashTable *server_sessions;

GdaClient *sussen_be_client;
GdaConnection *sussen_be_connection;

GladeXML *dsn_config_xml; 
GtkWidget *dsn_config_dialog;
GtkWidget *dsn_config_druid;

void load_gnome_frontend (void) {
	
    /* load the interface */
    mainwindow_xml = glade_xml_new(PACKAGE_DATA_DIR"/sussen.glade", "sussen_main_window", NULL);	
	main_window = glade_xml_get_widget (mainwindow_xml,"sussen_main_window");

    /* connect the signals in the interface */
    glade_xml_signal_autoconnect(mainwindow_xml); 

	/* Setup and re-draw tree widget */
	sensor_tree = glade_xml_get_widget (mainwindow_xml,"server_treeview");
	sensor_store = gtk_list_store_new (2, GDK_TYPE_PIXBUF, G_TYPE_STRING);
	gtk_tree_view_set_model (GTK_TREE_VIEW(sensor_tree), GTK_TREE_MODEL (sensor_store));
	{
		GtkCellRenderer *text_renderer, *pixbuf_renderer;
		GtkTreeViewColumn *column;

		pixbuf_renderer = gtk_cell_renderer_pixbuf_new ();
		column = gtk_tree_view_column_new_with_attributes ("Connected",pixbuf_renderer,
															"pixbuf",0,NULL);
		gtk_tree_view_append_column (GTK_TREE_VIEW (sensor_tree), column);
		
		text_renderer = gtk_cell_renderer_text_new ();
		column = gtk_tree_view_column_new_with_attributes ("Nessus Server",text_renderer,"text", 1,NULL);
		gtk_tree_view_column_set_sort_column_id (column, 1);
		gtk_tree_view_append_column (GTK_TREE_VIEW (sensor_tree), column);		
	}

	update_server_listview ();
	
	/*
	 * Configure scan_status_listview
	 */
	 
	GtkWidget *selected; 
	 
	scan_status_tree = glade_xml_get_widget (mainwindow_xml,"scan_status_listview");
	selected = GTK_WIDGET(gtk_tree_view_get_selection (GTK_TREE_VIEW(scan_status_tree)));
	gtk_tree_selection_set_mode (GTK_TREE_SELECTION(selected), GTK_SELECTION_MULTIPLE);
	scan_status_store = gtk_list_store_new (3,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING);
	gtk_tree_view_set_model (GTK_TREE_VIEW (scan_status_tree), GTK_TREE_MODEL (scan_status_store));
	{
		GtkTreeViewColumn *column;
		GtkCellRenderer *text_renderer;
		
		text_renderer = gtk_cell_renderer_text_new ();
		column = gtk_tree_view_column_new_with_attributes ("Hostname",text_renderer,
															"text",0,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 0);
		gtk_tree_view_append_column (GTK_TREE_VIEW (scan_status_tree), column);	
		
		text_renderer = gtk_cell_renderer_text_new ();
		column = gtk_tree_view_column_new_with_attributes ("Port Scan",text_renderer,
															"text",1,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 1);
		gtk_tree_view_append_column (GTK_TREE_VIEW (scan_status_tree), column);
		
		text_renderer = gtk_cell_renderer_text_new ();
		column = gtk_tree_view_column_new_with_attributes ("Attack",text_renderer,
															"text",2,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 2);
		gtk_tree_view_append_column (GTK_TREE_VIEW (scan_status_tree), column);		
	}	
	
	set_widgets_sensitive (FALSE,FALSE);
	
	GtkWidget *w;

	w = glade_xml_get_widget (mainwindow_xml, "scan_policy_radio");
	g_object_set_data(G_OBJECT(w), "scan_policy_switch", (void*)1);
	
	w = glade_xml_get_widget (mainwindow_xml, "scan_policy_use_current_radio");
	g_object_set_data(G_OBJECT(w), "scan_policy_switch", (void*)2);

	w = glade_xml_get_widget (mainwindow_xml, "hpaned1");
	gtk_paned_set_position (GTK_PANED(w), 275);
	
	gtk_window_resize (GTK_WINDOW(main_window), 800, 600);	
}

void config_dsn_cb (GtkWidget *w, gpointer user_data) {
	
	GdaDataSourceInfo *info = (GdaDataSourceInfo *)gnome_db_dsn_config_druid_get_dsn (GNOME_DB_DSN_CONFIG_DRUID(dsn_config_druid));
	gda_config_save_data_source (info->name, info->provider, info->cnc_string,
								 info->description, info->username, info->password);
	set_backend_xml (info->name, info->username, info->password);

    /* 
	 * FIXME: causes hang on gtk_widget_destroy () 
	 */
//	gtk_widget_destroy (dsn_config_dialog);	
	gtk_widget_hide (dsn_config_dialog);
}

gint main(gint argc, gchar *argv[]) {
	
	F_nessusd_running = 0;

	bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (PACKAGE);
	
	/* libxml */
	xmlKeepBlanksDefault(0);
		
	/* gnome */	
	gnome_program_init (PACKAGE,VERSION,LIBGNOMEUI_MODULE,argc,argv,
						 GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);

	/* gnome-db */
	gnome_db_init (PACKAGE, VERSION, argc, argv);
	sussen_be_client = gda_client_new ();
	
	/* init datalists */
	g_datalist_init (&server_preferences);
	g_datalist_init (&server_information);
	g_datalist_init (&plugin_depends);
	
	plugin_hash = g_hash_table_new (g_str_hash,g_str_equal);
	server_sessions = g_hash_table_new (g_str_hash,g_str_equal);

	if (!dot_sussen_there()) {
		create_dot_sussen ();
		create_default_reports_xml ();
//		create_default_cert_xml ();
		
		gboolean launch_dsn_druid = show_getting_started_druid ();
		if (launch_dsn_druid) {	
			
			dsn_config_xml = glade_xml_new(PACKAGE_DATA_DIR"/sussen.glade", "sussen_config_dsn_dialog", NULL);	
			dsn_config_dialog = glade_xml_get_widget (dsn_config_xml,"sussen_config_dsn_dialog");
			dsn_config_druid = glade_xml_get_widget (dsn_config_xml,"dbdsnconfigdruid1");
			
			g_signal_connect (GTK_OBJECT (dsn_config_druid),
							  "finished",
							  G_CALLBACK (config_dsn_cb),
					  			NULL);
			
			gtk_dialog_run (GTK_DIALOG (dsn_config_dialog));

		}
		sussen_be_connection = sussen_backend_init (sussen_be_client, TRUE);
	} else {
		sussen_be_connection = sussen_backend_init (sussen_be_client, FALSE);		
	}

	if (!sussen_be_connection) {
		sussen_error_dialog (_("fatal: Unable to make connection to database backend\n"));
		exit (-1);
	}	
	
	/*
  	 * Set up the main window
 	 */

	load_gnome_frontend ();
	gtk_main ();
			
	/* libxml/xslt */
	xsltCleanupGlobals();
	xmlCleanupParser();

	/* gnome-db */	
	gda_client_close_all_connections (sussen_be_client);
	g_object_unref (G_OBJECT(sussen_be_client));
	
	return 0;
}
