/* 
 * Copyright (C) 2003 Star Chamber Inc.
 *
 * Sussen - sussen-xml.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Author: Loren Bandiera <lorenb@starchamber.ca>
 */
 
#include "sussen.h"
#include "sussen-backend.h"
#include "sussen-fileio.h"
#include "sussen-xml.h"
#include <glib.h>
#include <libgnomedb/libgnomedb.h>

void create_default_reports_xml (void) {

	xmlDocPtr doc;
	xmlNodePtr child;
	gchar *buf;

	gchar *filename = get_reports_xml_filename ();

	doc = xmlNewDoc("1.0");
	doc->children = xmlNewDocNode (doc, NULL, "reports", NULL);
	
	child = xmlNewChild(doc->children, NULL, "report", NULL); 
	xmlSetProp (child, "name", "Raw Data");
	xmlSetProp (child, "file", "none");
	xmlSetProp (child, "format", "XML");

	child = xmlNewChild(doc->children, NULL, "report", NULL); 
	xmlSetProp (child, "name", "Vulnerability Assessment");
	buf = g_strdup_printf ("%s/va_report_html.xsl", PACKAGE_DATA_DIR);
	xmlSetProp (child, "file", buf);
	g_free (buf);
	xmlSetProp (child, "format", "XHTML");

	child = xmlNewChild(doc->children, NULL, "report", NULL); 
	xmlSetProp (child, "name", "Network Service Report");
	buf = g_strdup_printf ("%s/service_report_html.xsl", PACKAGE_DATA_DIR);
	xmlSetProp (child, "file", buf);
	g_free (buf);
	xmlSetProp (child, "format", "XHTML");	
	
	xmlSaveFile (filename,doc);	
	
	xmlFreeDoc(doc);
	g_free (filename);
}

void create_default_cert_xml (void) {

	xmlDocPtr doc;
	gchar *filename;
	
	gchar *tmp = get_dot_sussen_path ();
	filename = g_strdup_printf ("%s/server.cert.xml", tmp);

	doc = xmlNewDoc("1.0");
	doc->children = xmlNewDocNode (doc, NULL, "certs", NULL);

	xmlSaveFile (filename,doc);	
	
	xmlFreeDoc(doc);
	g_free (filename);
	g_free (tmp);
}

static gchar* getname (gchar *str) {

	gint i = 0, offset = 0, len = strlen(str);
	gchar *ret = g_malloc (len);
	
	for (i = 0; i < len; i++) {
		if (str[i] !=  '(' && str[i] != ' ') 
			ret[offset++] = str[i];
		else
			break;
	}

	ret[offset] = '\0';
	return ret;
}

gchar* getport (gchar *str) {

	gchar *port = strtok (str, "/");
	gchar *ret = g_strdup_printf ("%s", port);
	
	return ret;
}

gchar* getproto (gchar *str) {

	strtok (str, "/");
	gchar *protocol = strtok (NULL, "/");
	gchar *ret = g_strdup_printf ("%s", protocol);
	
	return ret;
}

xmlDocPtr sussen_backend_to_xml (GdaConnection *conn, const gchar *session_name) {
	
	xmlDocPtr doc;
	xmlNodePtr info_child;
	xmlNodePtr nessusd_child;
	xmlNodePtr host_child;
	xmlNodePtr date_child;
	xmlNodePtr results_child;
	xmlNodePtr result_child;
	xmlNodePtr host;
	xmlNodePtr ports;
	xmlNodePtr port;
	xmlNodePtr port_service;
	xmlNodePtr port_info;
	xmlNodePtr port_info_severity;
	xmlNodePtr port_info_id;
	xmlNodePtr port_info_data;
		
	doc = xmlNewDoc("1.0");
	doc->children = xmlNewDocNode (doc, NULL, "report", NULL);
	xmlSetProp (doc->children, "version", "1.0");
	
	info_child = xmlNewChild(doc->children, NULL, "info", NULL);
	nessusd_child = xmlNewChild (info_child, NULL, "nessusd", NULL);
	
	if (server_information) {
		xmlNodePtr child;
		
		child = xmlNewChild (nessusd_child, NULL, "version", (gchar *) g_datalist_get_data (&server_information,"server_info_nessusd_version"));
		child = xmlNewChild (nessusd_child, NULL, "libnasl", (gchar *) g_datalist_get_data (&server_information,"server_info_libnasl_version"));
		child = xmlNewChild (nessusd_child, NULL, "libnessus", (gchar *) g_datalist_get_data (&server_information,"server_info_libnessus_version"));
		child = xmlNewChild (nessusd_child, NULL, "thread", (gchar *) g_datalist_get_data (&server_information,"server_info_thread_manager"));
	} else {
		g_print ("sussen-xml: No nessusd informations.\n");
	}
	 
	host_child = xmlNewChild (info_child, NULL, "host", NULL);
	if (server_information) {
		xmlNodePtr child;

		child = xmlNewChild (host_child, NULL, "osname", (gchar *) g_datalist_get_data (&server_information,"server_info_os"));
		child = xmlNewChild (host_child, NULL, "osvers", (gchar *) g_datalist_get_data (&server_information,"server_info_os_version"));		
	}
	
	date_child = xmlNewChild (info_child, NULL, "date", NULL);

	gchar *sql = g_strdup_printf ("SELECT start_time FROM sessions WHERE session_id='%s'", session_name);
	GdaDataModel *date_dm = sussen_backend_raw (conn, sql);

	GdaRow *date_row = (GdaRow *) gda_data_model_get_row (date_dm, 0);
	GdaValue *date_value = (GdaValue*) gda_row_get_value (date_row, 0); 
	
	gchar *date_string = gda_value_stringify (date_value);	
	if (date_string) {
		xmlNodePtr child;
		
		child = xmlNewChild (date_child, NULL, "start",date_string);
	}
	g_free (sql);
	
	sql = g_strdup_printf ("SELECT end_time FROM sessions WHERE session_id='%s'", session_name);
	date_dm = sussen_backend_raw (conn, sql);
	date_row = (GdaRow *) gda_data_model_get_row (date_dm, 0);
	date_value = (GdaValue*) gda_row_get_value (date_row, 0); 
	date_string = gda_value_stringify (date_value);	
	if (date_string) {
		xmlNodePtr child;
		
		child = xmlNewChild (date_child, NULL, "end",date_string);
	}
	g_free (sql);

	results_child = xmlNewChild(doc->children, NULL, "results", NULL);

	gchar *network_sql = g_strdup_printf ("SELECT subnet FROM scan_data WHERE session_id='%s' GROUP BY subnet", session_name);
	GdaDataModel *network_dm = sussen_backend_raw (conn, network_sql);

	gint network_row_id;
	GdaRow *network_row;

	for (network_row_id = 0; network_row_id < gda_data_model_get_n_rows (network_dm); network_row_id++) {
	
		network_row = (GdaRow *) gda_data_model_get_row (network_dm, network_row_id);
		GdaValue *network_value = (GdaValue*) gda_row_get_value (network_row, 0); 
 		gchar *network_string = gda_value_stringify (network_value);
		
		gchar *host_sql = g_strdup_printf ("SELECT ip_address FROM scan_data WHERE session_id='%s' AND subnet='%s' GROUP BY ip_address", session_name,network_string);
		GdaDataModel *host_dm = sussen_backend_raw (conn, host_sql);

		gint host_row_id;
		GdaRow *host_row;

		for (host_row_id = 0; host_row_id < gda_data_model_get_n_rows (host_dm); host_row_id++) {
		
			host_row = (GdaRow *) gda_data_model_get_row (host_dm, host_row_id);
			GdaValue *host_value = (GdaValue *) gda_row_get_value (host_row, 0); 
 			gchar *host_string = gda_value_stringify (host_value);
			
			result_child = xmlNewChild (results_child, NULL, "result",NULL);					
			host = xmlNewChild (result_child, NULL, "host",NULL);
			xmlSetProp (host, "name", host_string);
			xmlSetProp (host, "ip", host_string);
					
			ports = xmlNewChild (result_child, NULL, "ports",NULL);
			gchar *port_sql = g_strdup_printf ("SELECT service FROM scan_data WHERE session_id='%s' AND subnet='%s' AND ip_address='%s' GROUP BY service", session_name,network_string,host_string);
			GdaDataModel *port_dm = sussen_backend_raw (conn, port_sql);

			gint port_row_id;
			GdaRow *port_row;

			for (port_row_id = 0; port_row_id < gda_data_model_get_n_rows (port_dm); port_row_id++) {
		
				port_row = (GdaRow *) gda_data_model_get_row (port_dm, port_row_id);
				GdaValue *port_value = (GdaValue *) gda_row_get_value (port_row, 0); 
	 			gchar *port_string = gda_value_stringify (port_value);
				
				gchar *port_protocol;
				gchar *port_name;
				gchar *port_number;
						
				port_number = getport (g_strdup(port_string));
				port_protocol = getproto (g_strdup(port_string));
				if (port_number && (atoi(port_number) > 0)) {							
					port = xmlNewChild (ports, NULL, "port",NULL);
					xmlSetProp (port, "protocol", port_protocol);
					xmlSetProp (port, "portid", port_number);							
				} else {
					port = xmlNewChild (ports, NULL, "port",NULL);
					xmlSetProp (port, "protocol", port_protocol);
				}
				
				g_free (port_protocol);
				g_free (port_number);

				port_name = getname (g_strdup(port_string));
				port_service = xmlNewChild (port, NULL, "service",NULL);
				xmlSetProp (port_service, "portid", port_name);
				xmlSetProp (port_service, "method", "nessus");
				xmlSetProp (port_service, "conf", "3");
				
				gchar *info_sql = g_strdup_printf ("SELECT risk_level,plugin_id,description FROM scan_data WHERE session_id='%s' AND subnet='%s' AND ip_address='%s' AND service='%s' AND type > 0", session_name,network_string,host_string,port_string);
				GdaDataModel *info_dm = sussen_backend_raw (conn, info_sql);

				gint info_row_id;
				GdaRow *info_row;
	
				for (info_row_id = 0; info_row_id < gda_data_model_get_n_rows (info_dm); info_row_id++) {
	
					info_row = (GdaRow *) gda_data_model_get_row (info_dm, info_row_id);
					GdaValue *info_value = (GdaValue*) gda_row_get_value (info_row, 0); 

 					gchar *info_string = gda_value_stringify (info_value);
					port_info = xmlNewChild (port, NULL, "information",NULL);
					port_info_severity = xmlNewChild (port_info, NULL, "severity",info_string);
			
					info_value = (GdaValue*) gda_row_get_value (info_row, 1); 
 					gchar *id_string = gda_value_stringify (info_value);
					port_info_id = xmlNewChild (port_info, NULL, "id",id_string);	

					info_value = (GdaValue*) gda_row_get_value (info_row, 2); 
 					gchar *data_string = gda_value_stringify (info_value);
					port_info_data = xmlNewChild (port_info, NULL, "data",data_string);
				}
				g_free (info_sql);				
			}	
			g_free (port_sql);
		}
		g_free (host_sql);
	}	
	g_free (network_sql);
	
	return doc;	
}
